/***************************************************************************
                          fleet.cpp  -  description
                             -------------------
    begin                : Tue Jan 22 2002
    copyright            : (C) 2002 by Michael Andreen
    email                : whale@linux.nu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fleet.h"

#include <iostream>
using namespace std;

//Static variables
map<string, vector<int> > Fleet::s_Races;
UnitList Fleet::s_Units;

Fleet::Fleet()
{
	m_iETA = 0;
	m_sRace = "Cathaar";
}

Fleet::~Fleet(){
}

//////////////////////////////////////////////////////////////////////////
//
void Fleet::setName(string sName)
{
	m_sName = sName;
}

//////////////////////////////////////////////////////////////////////////
//
string Fleet::name() const
{
	return m_sName;
}

//////////////////////////////////////////////////////////////////////////
//
/** This function first sets the race, then it iterates through the the 
 * s_Races and checks if it finds the race it returns true, if it reaches
 * the end without finding it it returns false.
 */
bool Fleet::setRace(string sRace)
{
	m_sRace = sRace;
	for (map<string, vector<int> >::iterator i = s_Races.begin(); i != s_Races.end(); i++)
	{
		if (m_sRace == (*i).first)
			return true;
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////
//
string Fleet::race() const
{
	return m_sRace;
}

//////////////////////////////////////////////////////////////////////////
//
/** This function iterates through m_Fleet and adds all numbers together to
 * produce a total.
 */
int Fleet::numberOfShips() const
{
	int total = 0;

	for (map<string, vector<int> >::const_iterator i = m_Fleet.begin(); i != m_Fleet.end(); ++i)
	{
		if (i->second.size() != 0)
			total += i->second[0];
	}

	return total;
}

//////////////////////////////////////////////////////////////////////////
//
void Fleet::setETA(int eta)
{
	m_iETA = eta;
}

//////////////////////////////////////////////////////////////////////////
//
int  Fleet::ETA() const
{
	return m_iETA;
}

//////////////////////////////////////////////////////////////////////////
//
void Fleet::setRaces(map<string, vector<int> >& races)
{
	s_Races = races;
}

//////////////////////////////////////////////////////////////////////////
//
void Fleet::setUnits(UnitList& units)
{
	s_Units = units;

	/*

	for (UnitList::iterator i = s_Units.begin(); i != s_Units.end(); i++)
	{
		cerr << s_Units[(*i).first].Name() << "\t\t"
			<< s_Units[(*i).first].race() <<"\t"
			<< s_Units[(*i).first].unitClass() << "\t"
			<< s_Units[(*i).first].target(0) << "\t"
			<< s_Units[(*i).first].target(1) << "\t"
			<< s_Units[(*i).first].target(2) << "\t"
			<< s_Units[(*i).first].initiative() << "\t"
			<< s_Units[(*i).first].agility() << "\t"
			<< s_Units[(*i).first].weaponSpeed() << "\t"
			<< s_Units[(*i).first].guns() << "\t"
			<< s_Units[(*i).first].power() << "\t"
			<< s_Units[(*i).first].armor() << "\t"
			<< s_Units[(*i).first].EMP() << "\t"
			<< s_Units[(*i).first].totRes() << "\t"
			<< s_Units[(*i).first].fuel() << "\t"
			<< s_Units[(*i).first].ETA() << "\t"
			<< s_Units[(*i).first].type() << endl;
	}
	*/
}

//////////////////////////////////////////////////////////////////////////
//
const map<string, vector<int> >& Fleet::Races()
{
	return s_Races;
}

//////////////////////////////////////////////////////////////////////////
//
const UnitList& Fleet::Units()
{
	return s_Units;
}

//////////////////////////////////////////////////////////////////////////
//
vector<int> Fleet::RacesAllowed() const
{
	return s_Races[m_sRace];
}

//////////////////////////////////////////////////////////////////////////
//
unsigned Fleet::score(int tick = 0) const
{
	unsigned tot_score = 0;

	for (FleetList::const_iterator i = m_Fleet.begin(); i != m_Fleet.end(); ++i)
	{
		  if (i->second.size() >= tick)
				break;
			tot_score += i->second[tick] * s_Units[i->first].totRes() / 10;
	}

	return tot_score;
}

//////////////////////////////////////////////////////////////////////////
//
void Fleet::setFleet(string unittype, int number)
{
	if (m_Fleet[unittype].size() == 0)
	{
		m_Fleet[unittype].push_back(number);
		return;
	}
	m_Fleet[unittype][0] = number;
}

//////////////////////////////////////////////////////////////////////////
//
int	 Fleet::fleet(string unittype, int tick = 0)
{
	if (m_Fleet[unittype].size() == 0)
		return 0;

	return m_Fleet[unittype][tick];
}

	
