/***************************************************************************
                          fleet.h  -  description
                             -------------------
    begin                : Tue Jan 22 2002
    copyright            : (C) 2002 by Michael Andreen
    email                : whale@linux.nu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FLEET_H
#define FLEET_H

#include <map>
#include <ext/hash_map>
#include <string>
#include <vector>

#include "unittype.h"
 template<typename T> class MyComp { public: bool operator()(T,T) { return false; } };
typedef std::map<std::string, std::vector<int> > FleetList;
typedef std::map<std::string, UnitType>	UnitList;
typedef std::map<std::string, std::vector<int> > RaceList;
//!This is the engine for the whole battlesystem.
/**This is the engine for the whole battlesystem.
  *	One of the few parts that I plan make fully portable.
  *\todo LOTS (I think ;)
  *@author Michael Andreen
  */
class Fleet 
{
public: 
	Fleet();
	virtual ~Fleet();

	/**Sets the name that represents this fleet. Might be different a name
	 * like foobar or some coordinates like 1:1:1. The name doesn't have to be unique,
	 * since it's up to the rest of the program to handle that part.
	 * \see Name
	 */
	void   setName(std::string sName);
	/**Returns the name of this fleet.
	 * \see setName
	 */
	std::string name() const;

	/**The race string decides what type of ships this fleet can have.
	 * The values must be feeded into this class.
	 * \param sRace This is just the name of the race. It's case-sensitive.
	 * \return If the race is available this function returns true, if not false is returned. The race is set in both cases though.
	 * \see Race
	 */
	bool   setRace(std::string sRace);
	/**Just returns what race this fleet belongs to.. 
	 * \return The race, represented as a string.
	 * \see setRace
	 */
	std::string race() const;

	std::vector<int> RacesAllowed() const;

	/**Returns the total number of ships in this fleet
	 */
	int numberOfShips() const;

	/**Sets the estimated time of arrival. The time as a single integer,
	 * in relation to the current time. For example if the current time is
	 * 10, and the arrival is at 12, then the eta is 2.
	 */
	void setETA(int eta);
	/**Return the estimated time of arrival. It's counted from the current time (tick).
	 */
	int  ETA() const;


	/**Returns the score. This value is the total resources spent on this fleet
	 * devided with 10.
	 * \param tick tells the function what tick you want the score from. 0 is 
	 * initial score before the fleet has landed.
	 */
	unsigned score(int tick = 0) const;

	void setFleet(std::string unittype, int number);
	int	 fleet(std::string unittype, int tick = 0);
	
	static void setRaces(RaceList& races);
	static void setUnits(UnitList& units);

	static const RaceList& Races();
	static const UnitList& Units();

protected:
	std::string	m_sName;
	std::string	m_sRace;
	int         m_iETA;
	FleetList	m_Fleet;

	static UnitList s_Units;
	static RaceList s_Races;
};

#endif
